<?php
namespace Pinpoint\DeliveryDate\Controller\Adminhtml\Disallowed;

use Magento\Backend\App\Action;
use Pinpoint\DeliveryDate\Model\Page;
use Magento\Framework\App\Request\DataPersistorInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;

/**
 * Class Save
 * @package Pinpoint\DeliveryDate\Controller\Adminhtml\Disallowed
 */
class Save extends \Magento\Backend\App\Action
{
    /**
     * Authorization level of a basic admin session
     *
     * @see _isAllowed()
     */
    const ADMIN_RESOURCE = 'Pinpoint_DeliveryDate::delivery_dates';

    /**
     * @var DataPersistorInterface
     */
    protected $dataPersistor;

    /**
     * @var \Pinpoint\DeliveryDate\Model\DisallowedDateRepository
     */
    protected $disallowedDateRepository;

    protected $disallowedDateFactory;


    public function __construct(
        Action\Context $context,
        DataPersistorInterface $dataPersistor,
        \Pinpoint\DeliveryDate\Model\DisallowedDateRepository $disallowedDate,
        \Pinpoint\DeliveryDate\Model\DisallowedDateFactory $disallowedDateFactory
    ) {
        $this->dataPersistor = $dataPersistor;
        $this->disallowedDateRepository = $disallowedDate;
        $this->disallowedDateFactory = $disallowedDateFactory;
        parent::__construct($context);
    }

    /**
     * Check the permission to run it
     *
     * @return boolean
     */
    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('Pinpoint_DeliveryDate::delivery_dates');
    }

    /**
     * Save action
     *
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $data = $this->getRequest()->getPostValue();
        /** @var \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
        $resultRedirect = $this->resultRedirectFactory->create();
        if ($data) {
            if (empty($data['id'])) {
                $data['id'] = null;
            }

            $id = $this->getRequest()->getParam('id');
            $model = $this->disallowedDateFactory->create();
            if ($id) {
                try {
                    $model = $this->disallowedDateRepository->getById($id);
                } catch (NoSuchEntityException $ex) {
                    unset($ex);
                }
            }

            $model->setData($data);

            try {
                $model->save();
                $this->messageManager->addSuccessMessage(__('Date has been saved.'));
                $this->dataPersistor->clear('pinpoint_deliverydate_disalloweddate');
                if ($this->getRequest()->getParam('back')) {
                    return $resultRedirect->setPath('*/*/edit', ['id' => $model->getId(), '_current' => true]);
                }
                return $resultRedirect->setPath('*/*/list');
            } catch (LocalizedException $e) {
                $this->messageManager->addErrorMessage($e->getMessage());
            } catch (\Exception $e) {
                $this->messageManager->addExceptionMessage($e, __('Something went wrong while saving the date.'));
            }

            $this->dataPersistor->set('pinpoint_deliverydate_disalloweddate', $data);
            return $resultRedirect->setPath('*/*/edit', ['id' => $this->getRequest()->getParam('id')]);
        }
        return $resultRedirect->setPath('*/*/list');
    }    
}
