<?php
namespace Pinpoint\DeliveryDate\Controller\Adminhtml\Estimated;

use Magento\Backend\App\Action;
use Pinpoint\DeliveryDate\Model\Page;
use Magento\Framework\App\Request\DataPersistorInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;

/**
 * Class Save
 * @package Pinpoint\DeliveryDate\Controller\Adminhtml\Estimated
 */
class Save extends \Magento\Backend\App\Action
{
    /**
     * Authorization level of a basic admin session
     *
     * @see _isAllowed()
     */
    const ADMIN_RESOURCE = 'Pinpoint_DeliveryDate::delivery_dates';

    /**
     * @var DataPersistorInterface
     */
    protected $dataPersistor;

    /**
     * @var \Pinpoint\DeliveryDate\Model\EstimatedDateRepository
     */
    protected $estimatedDateRepository;

    /**
     * @var \Pinpoint\DeliveryDate\Model\EstimatedDateFactory
     */
    protected $estimatedDateFactory;

    /**
     * Save constructor.
     * @param Action\Context $context
     * @param DataPersistorInterface $dataPersistor
     * @param \Pinpoint\DeliveryDate\Model\EstimatedDateRepository $estimatedDate
     * @param \Pinpoint\DeliveryDate\Model\EstimatedDateFactory $estimatedDateFactory
     */
    public function __construct(
        Action\Context $context,
        DataPersistorInterface $dataPersistor,
        \Pinpoint\DeliveryDate\Model\EstimatedDateRepository $estimatedDate,
        \Pinpoint\DeliveryDate\Model\EstimatedDateFactory $estimatedDateFactory
    ) {
        $this->dataPersistor = $dataPersistor;
        $this->estimatedDateRepository = $estimatedDate;
        $this->estimatedDateFactory = $estimatedDateFactory;
        parent::__construct($context);
    }

    /**
     * Check the permission to run it
     *
     * @return boolean
     */
    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('Pinpoint_DeliveryDate::delivery_dates');
    }

    /**
     * Save action
     *
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $data = $this->getRequest()->getPostValue();
        /** @var \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
        $resultRedirect = $this->resultRedirectFactory->create();
        if ($data) {
            if (empty($data['id'])) {
                $data['id'] = null;
            }

            $model = $this->estimatedDateFactory->create();
            $id = $this->getRequest()->getParam('id');

            if ($id) {
                try {
                    $model = $this->estimatedDateRepository->getById($id);
                } catch (NoSuchEntityException $ex) {
                    unset($ex);
                }
            }

            $data['shipping_method_code'] = implode(',', $data['shipping_method_code']);
            if (isset($data['excluding_days']) && is_array($data['excluding_days']))
                $data['excluding_days'] = implode(',', $data['excluding_days']);
            else
                $data['excluding_days'] = '';

            $model->setData($data);

            try {
                $model->save();
                $this->messageManager->addSuccess(__('Estimation Saved.'));
                $this->dataPersistor->clear('pinpoint_deliverydate_estimateddate');
                if ($this->getRequest()->getParam('back')) {
                    return $resultRedirect->setPath('*/*/edit', ['id' => $model->getId(), '_current' => true]);
                }
                return $resultRedirect->setPath('*/*/list');
            } catch (LocalizedException $e) {
                $this->messageManager->addError($e->getMessage());
            } catch (\Exception $e) {
                $this->messageManager->addException($e, __('Something went wrong while saving the data.'));
            }

            $this->dataPersistor->set('pinpoint_deliverydate_estimateddate', $data);
            return $resultRedirect->setPath('*/*/edit', ['id' => $this->getRequest()->getParam('id')]);
        }
        return $resultRedirect->setPath('*/*/list');
    }    
}
