<?php
namespace Pinpoint\DeliveryDate\Model;

use Pinpoint\DeliveryDate\Api\DisallowedDateRepositoryInterface;
use Pinpoint\DeliveryDate\Api\Data\DisallowedDateInterface;
use Pinpoint\DeliveryDate\Model\DisallowedDateFactory;
use Pinpoint\DeliveryDate\Model\ResourceModel\DisallowedDate\CollectionFactory;

use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Exception\CouldNotDeleteException;
use Magento\Framework\Api\SearchResults;

/**
 * Class DisallowedDateRepository
 * @package Pinpoint\DeliveryDate\Model
 */
class DisallowedDateRepository implements \Pinpoint\DeliveryDate\Api\DisallowedDateRepositoryInterface
{
    /**
     * @var \Pinpoint\DeliveryDate\Model\DisallowedDateFactory
     */
    protected $objectFactory;

    /**
     * @var CollectionFactory
     */
    protected $collectionFactory;

    /**
     * @var SearchResults
     */
    private $searchResultsFactory;

    /**
     * DisallowedDateRepository constructor.
     * @param \Pinpoint\DeliveryDate\Model\DisallowedDateFactory $objectFactory
     * @param CollectionFactory $collectionFactory
     * @param SearchResults $searchResultsFactory
     */
    public function __construct(
        DisallowedDateFactory $objectFactory,
        CollectionFactory $collectionFactory,
        SearchResults $searchResultsFactory       
    ) {
        $this->objectFactory        = $objectFactory;
        $this->collectionFactory    = $collectionFactory;
        $this->searchResultsFactory = $searchResultsFactory;
    }

    /**
     * @param DisallowedDateInterface $object
     * @return DisallowedDateInterface
     * @throws CouldNotSaveException
     */
    public function save(DisallowedDateInterface $object)
    {
        try {
            $object->save();
        } catch(\Exception $e) {
            throw new CouldNotSaveException(__($e->getMessage()));
        }
        return $object;
    }

    /**
     * @param $id
     * @return mixed
     * @throws NoSuchEntityException
     */
    public function getById($id)
    {
        $object = $this->objectFactory->create();
        $object->load($id);
        if (!$object->getId()) {
            throw new NoSuchEntityException(__('Object with id "%1" does not exist.', $id));
        }
        return $object;        
    }

    /**
     * @param DisallowedDateInterface $object
     * @return bool
     * @throws CouldNotDeleteException
     */
    public function delete(DisallowedDateInterface $object)
    {
        try {
            $object->delete();
        } catch (\Exception $exception) {
            throw new CouldNotDeleteException(__($exception->getMessage()));
        }
        return true;    
    }

    /**
     * @param $id
     * @return bool
     */
    public function deleteById($id)
    {
        return $this->delete($this->getById($id));
    }

    /**
     * @param SearchCriteriaInterface $criteria
     * @return mixed
     */
    public function getList(SearchCriteriaInterface $criteria)
    {
        $this->searchResultsFactory->setSearchCriteria($criteria);  
        $collection = $this->collectionFactory->create();
        foreach ($criteria->getFilterGroups() as $filterGroup) {
            $fields = [];
            $conditions = [];
            foreach ($filterGroup->getFilters() as $filter) {
                $condition = $filter->getConditionType() ? $filter->getConditionType() : 'eq';
                $fields[] = $filter->getField();
                $conditions[] = [$condition => $filter->getValue()];
            }
            if ($fields) {
                $collection->addFieldToFilter($fields, $conditions);
            }
        }  
        $searchResults->setTotalCount($collection->getSize());
        $sortOrders = $criteria->getSortOrders();
        if ($sortOrders) {
            /** @var SortOrder $sortOrder */
            foreach ($sortOrders as $sortOrder) {
                $collection->addOrder(
                    $sortOrder->getField(),
                    ($sortOrder->getDirection() == SortOrder::SORT_ASC) ? 'ASC' : 'DESC'
                );
            }
        }
        $collection->setCurPage($criteria->getCurrentPage());
        $collection->setPageSize($criteria->getPageSize());
        $objects = [];                                     
        foreach ($collection as $objectModel) {
            $objects[] = $objectModel;
        }
        $searchResults->setItems($objects);
        return $searchResults;        
    }
}