<?php
namespace Pinpoint\DeliveryDate\Setup;


/**
 * Class InstallSchema
 * @package Pinpoint\DeliveryDate\Setup
 */
class InstallSchema implements \Magento\Framework\Setup\InstallSchemaInterface
{
    /**
     * @param \Magento\Framework\Setup\SchemaSetupInterface $setup
     * @param \Magento\Framework\Setup\ModuleContextInterface $context
     */
    public function install(\Magento\Framework\Setup\SchemaSetupInterface $setup, \Magento\Framework\Setup\ModuleContextInterface $context)
    {
        $installer = $setup;
        $installer->startSetup();

        $this->createEstimatedDatesTable($setup);
        $this->createDisallowedDatesTable($setup);

        $installer->endSetup();
    }

    /**
     * @param \Magento\Framework\Setup\SchemaSetupInterface $setup
     */
    protected function createEstimatedDatesTable(\Magento\Framework\Setup\SchemaSetupInterface $setup)
    {
        $installer = $setup;
        $tableName = $installer->getTable('pinpoint_deliverydate_estimated_dates');

        if ($installer->getConnection()->isTableExists($tableName) != true) {
            $table = $installer->getConnection()->newTable(
                $installer->getTable('pinpoint_deliverydate_estimated_dates')
            )->addColumn(
                'id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                ['identity' => true, 'nullable' => false, 'primary' => true, 'unsigned' => true,],
                'ID'
            )->addColumn(
                'shipping_method_code',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                255,
                ['nullable' => false,],
                'Shipping method code'
            )->addColumn(
                'estimated_delivery_days',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                255,
                ['nullable' => false,],
                'Delivery date estimation'
            )->addColumn(
                'store_id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                255,
                ['nullable' => false,],
                'Store ID'
            )->addColumn(
                'excluding_weekdays',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                255,
                ['nullable' => false,],
                'Store ID'
            )->addColumn(
                'update_time',
                \Magento\Framework\DB\Ddl\Table::TYPE_TIMESTAMP,
                null,
                ['nullable' => false, 'default' => \Magento\Framework\DB\Ddl\Table::TIMESTAMP_INIT_UPDATE,],
                'Modification Time'
            );

            $installer->getConnection()->createTable($table);
        }
    }

    /**
     * @param \Magento\Framework\Setup\SchemaSetupInterface $setup
     */
    protected function createDisallowedDatesTable(\Magento\Framework\Setup\SchemaSetupInterface $setup)
    {
        $installer = $setup;
        $tableName = $installer->getTable('pinpoint_deliverydate_disallowed_dates');

        if ($installer->getConnection()->isTableExists($tableName) != true) {
            $table = $installer->getConnection()->newTable(
                $installer->getTable('pinpoint_deliverydate_disallowed_dates')
            )->addColumn(
                'id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                ['identity' => true, 'nullable' => false, 'primary' => true, 'unsigned' => true,],
                'ID'
            )->addColumn(
                'disallowed_date',
                \Magento\Framework\DB\Ddl\Table::TYPE_DATE,
                255,
                ['nullable' => false,],
                'Disallowed Date'
            )->addColumn(
                'disallowed_date_name',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                255,
                ['nullable' => false,],
                'Delivery date name'
            )->addColumn(
                'store_id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                255,
                ['nullable' => false,],
                'Store ID'
            )->addColumn(
                'update_time',
                \Magento\Framework\DB\Ddl\Table::TYPE_TIMESTAMP,
                null,
                ['nullable' => false, 'default' => \Magento\Framework\DB\Ddl\Table::TIMESTAMP_INIT_UPDATE,],
                'Modification Time'
            );

            $installer->getConnection()->createTable($table);
        }
    }
}
